package gov.usgs.earthquake.nshmp.netcdf.www;

import static com.google.common.base.CaseFormat.UPPER_CAMEL;
import static com.google.common.base.CaseFormat.UPPER_UNDERSCORE;
import static gov.usgs.earthquake.nshmp.internal.www.WsUtils.EnumSerializer;
import static gov.usgs.earthquake.nshmp.internal.www.WsUtils.NaNSerializer;

import java.util.logging.Logger;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.internal.www.NshmpMicronautServlet.UrlHelper;
import gov.usgs.earthquake.nshmp.internal.www.Response;
import gov.usgs.earthquake.nshmp.internal.www.meta.Status;
import gov.usgs.earthquake.nshmp.netcdf.NshmGroup;
import gov.usgs.earthquake.nshmp.netcdf.SiteClass;

import io.micronaut.http.HttpResponse;

public class NetcdfWsUtils {

  static final Gson GSON;

  static {
    GSON = new GsonBuilder()
        .registerTypeAdapter(Imt.class, new EnumSerializer<Imt>())
        .registerTypeAdapter(NshmGroup.class, new EnumSerializer<NshmGroup>())
        .registerTypeAdapter(SiteClass.class, new EnumSerializer<SiteClass>())
        .registerTypeAdapter(Double.class, new NaNSerializer())
        .disableHtmlEscaping()
        .serializeNulls()
        .setPrettyPrinting()
        .create();
  }

  public static HttpResponse<String> handleError(
      Throwable e,
      String name,
      Logger logger,
      UrlHelper urlHelper) {
    var msg = e.getMessage() + " (see logs)";
    var svcResponse = new Response<>(Status.ERROR, name, urlHelper.url, msg, urlHelper);
    var response = GSON.toJson(svcResponse);
    logger.severe(name + " -\n" + response);
    e.printStackTrace();
    return HttpResponse.serverError(response);
  }

  static enum Key {
    LATITUDE,
    LONGITUDE,
    SITE_CLASS,
    IMT;

    @Override
    public String toString() {
      return UPPER_UNDERSCORE.to(UPPER_CAMEL, name());
    }
  }
}
